<?php
#############################################
#                                           #
#  ████  █████   ████  ██████  ████  ██████ #
# ██  ██ ██  ██ ██  ██ ██       ██   █ ██ █ #
# ██     ██  ██ ██  ██ ██       ██     ██   #
# ██ ███ █████  ██████ █████    ██     ██   #
# ██  ██ ██  ██ ██  ██ ██       ██     ██   #
# ██  ██ ██  ██ ██  ██ ██       ██     ██   #
#  ████  ██  ██ ██  ██ ██      ████   ████  #
#                                           #
#############################################

# CETTE API UTILISE phpGraph de mon copain JerryWham (bisou copain :-*)
# http://jerrywham.github.io/phpGraph/
# https://www.ecyseo.net/article29/phpgraph-afficher-des-graphiques-svg-grace-a-php
include('phpGraph.php');
include('zip_class.php');

define('DEFAULT_WIDTH',512);
define('DEFAULT_HEIGHT',256);
define('VERSION','1.12e');
$options=[];
$svg=new phpGraph();

function has_string_keys(array $array) {
  return count(array_filter(array_keys($array), 'is_string')) > 0;
}
if (!is_dir('generated')){mkdir('generated');}


if (isset($_GET['help'])){
	exit(nl2br(file_get_contents('readme.html')));
}

if (isset($_GET['version'])){
	exit(VERSION);
}

if (isset($_GET['download'])){
	$file='GrafitAPI_v'.VERSION.'.zip';
	if (is_file($file)){
		header("Location: ".$file);
		exit;
	}
	$zip=new zip();
	$files=glob('*.*');
	$files=array_reverse(array_reverse($files));
	# delete zip file from list
	foreach ($files as $key => $value) {
		if (stripos($value,'.zip')===false){
			$f[$key]=$value;
		}
	}
	$files=$f;
	$zip->setSource('.');
    $zip->setFilesList($files);
    $zip->setDestination($file);
    $zip->create();
    $zip->download();
    exit();
}


if (!empty($_GET['data'])){

	$filename=hash('sha1', $_SERVER['QUERY_STRING']).'.svg';
	$generated_picture_path='generated/'.$filename;
	# ne pas régénérer les images déjà générées
	if (is_file($generated_picture_path) && !isset($_GET['force'])){
		header('Content-type: image/svg+xml');
		exit(file_get_contents($generated_picture_path));
	}

	$data=json_decode($_GET['data'],true);
	if (is_array($data)) {
		foreach ($data as $key => $value) {
			if (is_array($value)) {
				$options['multi'] = true;
			}
		}
	}
	if (!empty($data[0])||isset($_GET['hidekeys'])){
		# if array is not associative, no need to display the keys (0,1,2,3,4,5,6 etc)
		$svg->setCss('g.graph-x{display:none}');
		$newdata=[];
		foreach ($data as $key => $value) {
			$newdata[" $key "]=floatval($value);
		}
		$data=$newdata;
	}
	//$data=array_map(function($v){return floatval($v);}, $data);
	$nb=count($data);
	$moymed='';


	if (!empty($_GET['legend'])){
		$options['legends'][]=strip_tags($_GET['legend']).' ';
	}else{
		$options['legends']=null;
	}

	if (isset($_GET['moyenne'])){
		$moyenne=round(array_sum($data)/$nb,2);
		$moymed='<tspan class="graph-average">Moyenne: '.$moyenne. '</tspan>';
		$options['average']=true;
	}else{
		$moymed='';
	}
	if (isset($_GET['mediane'])){
		$sorted_data=$data;
		sort($sorted_data);

		if ($nb%2==0){
			$k=$nb/2;
			$mediane=($sorted_data[$k]+$sorted_data[$k-1])/2;
		}else{
			$mediane=$sorted_data[($nb)/2];
		}
		if(!empty($moymed)){$moymed.=' | ';}
		$moymed.=' <tspan class="graph-median">Médiane: '.$mediane. '</tspan>';
		$options['median']=true;
	}
	if (!empty($moymed)){
		$options['legends'][]=$moymed;
	}


	if (isset($_GET['w'])){
		$options['width']=intval($_GET['w']);
	}
	if (isset($_GET['h'])){
		$options['height']=intval($_GET['h']);
	}

	if (isset($_GET['title'])){
		$options['title']=strip_tags($_GET['title']);
	}



	if (isset($_GET['lines'])||(!empty($_GET['type'])&&$_GET['type']=='lines')){
		$options['type']='line';
		$options['filled']=false;
	}elseif (isset($_GET['filled'])||(!empty($_GET['type'])&&$_GET['type']=='filled')){
		$options['type']='line';
	}elseif (isset($_GET['pie'])){
		$options['diskLegends']=true;
		$options['type']='pie';
	}elseif (isset($_GET['ring'])){
		$options['diskLegends']=true;
		$options['type']='ring';
	}else{
		$options['type']='bar';
	}
	if (!empty($_GET['background'])&&preg_match('#[0-9a-fA-F]{6}#', $_GET['background'])){
		$options['background']='#'.$_GET['background'];
	}
	if (!empty($_GET['color'])&&preg_match('#[0-9a-fA-F]{6}#', $_GET['color'])){
		$options['stroke']='#'.$_GET['color'];
	}

	if (!empty($_GET['steps'])){
		$options['steps']=floatval($_GET['steps']);
	} else {
		if (!isset($options['multi'])) {
			# calculate steps
			$max=max($data);
			$nb=strlen("$max");

			$nb=round($max - $nb);
			$options['steps']=($nb*10)/100;
		}
	}
	if (isset($options['multi'])) {
		$type = $options['type'];
		$stroke = $options['stroke'];
		$options['type'] = array();
		$options['stroke'] = array();
		$i = 0;
		foreach ($data as $line => $value) {
			$options['type'][$line] = $type;
			if ($i == 0) {
				$options['stroke'][$line] = $stroke;
				$i++;
			}
		}
	}

	if (isset($_GET['hidevalues'])){
		$options['values']=false;
		$options['diskLegends']=false;
	}
	$options['tooltips']=true;
	$options['opacity']=0.7;

	header('Content-type: image/svg+xml');
	//header('Content-Disposition: attachment; filename="'.$filename.'"');
	$graph=$svg->draw($data, $options);

	if (!isset($_GET['dontsave'])){
		file_put_contents($generated_picture_path,$graph);
	}
	exit($svg->draw($data, $options));
}
include('frontend.php');
